from django.db.models.signals import class_prepared
from stream_django.conf import FEED_MANAGER_CLASS
from stream_django.conf import DISABLE_MODEL_TRACKING
from stream_django.utils import get_class_from_string


feed_manager_class = get_class_from_string(FEED_MANAGER_CLASS)
feed_manager = feed_manager_class()

if not DISABLE_MODEL_TRACKING:
    class_prepared = class_prepared.connect(feed_manager.bind_model)



from django import template
from django.template import Context, loader


register = template.Library()


def render_activity(context, activity, template_prefix=''):
    if template_prefix != '':
        template_prefix = '%s_' % template_prefix
    if 'activities' in activity:
        template_name = "activity/aggregated/%s%s.html" % (template_prefix, activity['verb'])
    else:
        template_name = "activity/%s%s.html" % (template_prefix, activity['verb'])
    tmpl = loader.get_template(template_name)
    context['activity'] = activity
    context = Context(context)
    return tmpl.render(context)


register.simple_tag(takes_context=True)(render_activity)



from stream_django.client import stream_client


def stream(request):
    context = {}
    context['STREAM_API_KEY'] = stream_client.api_key
    context['STREAM_APP_ID'] = stream_client.app_id
    return context



from stream_django import conf
import os
import stream
from django.core.exceptions import ImproperlyConfigured

if conf.API_KEY and conf.API_SECRET:
    stream_client = stream.connect(
        conf.API_KEY, conf.API_SECRET)
else:
    stream_client = stream.connect()

if os.environ.get('STREAM_URL') is None and not(conf.API_KEY and conf.API_SECRET):
    raise ImproperlyConfigured('Stream credentials are not set in your settings')



import os
from setuptools import setup


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


test_requirements = [
    'django>=1.5',
    'httpretty',
    'coverage'
]

setup(
    name='stream-django',
    version='1.0.4',
    packages=['stream_django'],
    include_package_data=True,
    install_requires=[
        'stream-python>=2.0.0',
        'pytz'
    ],
    tests_require=test_requirements,
    setup_requires=test_requirements,
    test_suite='runtests.runtests',
    license='BSD License',
    description='A Django app to build activity, news and notification feeds.',
    long_description=README,
    url='https://getstream.io/',
    author='Tommaso Barbugli',
    author_email='tbarbugli@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from activity import Activity
from django.db.models.signals import post_delete, post_save
from stream_django.client import stream_client
from stream_django import conf


class FeedManager(object):

    def __init__(self):
        self.notification_feed = conf.NOTIFICATION_FEED
        self.user_feed = conf.USER_FEED
        self.news_feeds = conf.NEWS_FEEDS

    def get_user_feed(self, user_id, feed_type=None):
        if feed_type is None:
            feed_type = self.user_feed
        feed = stream_client.feed(feed_type, user_id)
        return feed

    def get_notification_feed(self, user_id):
        return stream_client.feed(self.notification_feed, user_id)

    def get_actor_feed(self, instance=None):
        if instance.activity_author_feed is not None:
            return instance.activity_author_feed
        else:
            return self.user_feed

    def follow_user(self, user_id, target_user_id):
        news_feeds = self.get_news_feeds(user_id)
        target_feed = self.get_user_feed(target_user_id)
        for feed in news_feeds.values():
            feed.follow(target_feed.slug, target_feed.user_id)

    def unfollow_user(self, user_id, target_user_id):
        news_feeds = self.get_news_feeds(user_id)
        target_feed = self.get_user_feed(target_user_id)
        for feed in news_feeds.values():
            feed.unfollow(target_feed.slug, target_feed.user_id)

    def get_feed(self, feed, user_id):
        return stream_client.feed(feed, user_id)

    def get_news_feeds(self, user_id):
        feeds = {}
        for feed in self.news_feeds:
            feeds[feed] = self.get_feed(feed, user_id)
        return feeds

    def activity_created(self, sender, instance, created, **kwargs):
        if created:
            activity = instance.create_activity()
            feed_type = self.get_actor_feed(instance)
            feed = self.get_feed(feed_type, instance.activity_actor_id)
            result = feed.add_activity(activity)
            return result

    def activity_delete(self, sender, instance, **kwargs):
        feed_type = self.get_actor_feed(instance)
        feed = self.get_feed(feed_type, instance.activity_actor_id)
        result = feed.remove_activity(foreign_id=instance.activity_foreign_id)
        return result

    def bind_model(self, sender, **kwargs):
        if issubclass(sender, (Activity, )):
            post_save.connect(self.activity_created, sender=sender)
            post_delete.connect(self.activity_delete, sender=sender)



from django.template.defaultfilters import slugify
from django.utils.timezone import is_aware
from django.utils.timezone import make_naive
import pytz


def model_content_type(cls):
    return '%s.%s' % (cls._meta.app_label, cls._meta.object_name)


def create_reference(reference):
    return create_model_reference(reference)
    # if isinstance():
    # return reference


def create_model_reference(model_instance):
    '''
    creates a reference to a model instance that can be stored in activities

    >>> from core.models import Like
    >>> like = Like.object.get(id=1)
    >>> create_reference(like)
    core.Like:1

    '''
    content_type = model_content_type(model_instance.__class__)
    content_id = model_instance.pk
    return '%s:%s' % (content_type, content_id)


class Activity(object):
    
    @property
    def activity_author_feed(self):
        '''
        The name of the feed where the activity will be stored; this is normally
        used by the manager class to determine if the activity should be stored elsewehere than
        settings.USER_FEED
        '''
        pass

    @classmethod
    def activity_related_models(cls):
        '''
        Use this hook to setup related models to load during enrichment.
        It must return None or a list of relationships see Django select_related for reference
        '''
        pass

    @property
    def extra_activity_data(self):
        '''
        Use this hook to store extra data in activities.
        If you need to store references to model instances you should use create_model_refeactivity_rence

        eg:
            @property
            def activity_extra_activity_data(self):
                dict('parent_user'=create_reference(self.parent_user))
        '''
        pass

    @property
    def activity_actor_attr(self):
        '''
        Returns the model instance field that references the activity actor
        '''
        return self.user

    @property
    def activity_actor_id(self):
        return self.activity_actor_attr.pk

    @property
    def activity_actor(self):
        return create_reference(self.activity_actor_attr)

    @property
    def activity_verb(self):
        model_name = slugify(self.__class__.__name__)
        return model_name
    
    @property
    def activity_object(self):
        return create_reference(self)

    @property
    def activity_foreign_id(self):
        return self.activity_object

    @property
    def activity_time(self):
        atime = self.created_at
        if is_aware(self.created_at):
            atime = make_naive(atime, pytz.utc)
        return atime
    
    @property
    def activity_notify(self):
        pass
    
    def create_activity(self):
        extra_data = self.extra_activity_data
        if not extra_data:
            extra_data = {}
        
        to = self.activity_notify
        if to:
            extra_data['to'] = [f.id for f in to]
        
        activity = dict(
            actor=self.activity_actor,
            verb=self.activity_verb,
            object=self.activity_object,
            foreign_id=self.activity_foreign_id,
            time=self.activity_time,
            **extra_data
        )
        return activity



from django.conf import settings


API_KEY = getattr(settings, 'STREAM_API_KEY', None)
API_SECRET = getattr(settings, 'STREAM_API_SECRET', None)

FEED_MANAGER_CLASS = getattr(settings, 'STREAM_FEED_MANAGER_CLASS',
    'stream_django.managers.FeedManager'
)

USER_FEED = getattr(settings, 'STREAM_USER_FEED', 'user')
NEWS_FEEDS = getattr(settings, 'STREAM_NEWS_FEEDS',
    {'flat':'flat', 'aggregated':'aggregated'}
)
NOTIFICATION_FEED = getattr(settings, 'STREAM_PERSONAL_FEED', 'notification')

DISABLE_MODEL_TRACKING = getattr(settings, 'STREAM_DISABLE_MODEL_TRACKING', False)
